"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startServer = void 0;
const maishu_node_web_server_1 = require("maishu-node-web-server");
const maishu_nws_js_1 = require("maishu-nws-js");
const json5_processor_1 = require("./processors/json5-processor");
const less_processor_1 = require("./processors/less-processor");
const errors = __importStar(require("./errors"));
const fs = __importStar(require("fs"));
const load_plugins_1 = require("./load-plugins");
const maishu_nws_mvc_1 = require("maishu-nws-mvc");
function startServer(settings, mode) {
    let packagePath = "../package.json";
    let pkg = require(packagePath);
    let logger = maishu_node_web_server_1.getLogger(pkg.name, settings.log?.level);
    mode = mode || "mvc";
    logger.info(`${startServer.name}: Current mode is ${mode}.`);
    if (settings.websiteDirectory == null)
        throw errors.arugmentFieldNull("rootDirectory", "settings");
    if (typeof settings.websiteDirectory == "string" && !fs.existsSync(settings.websiteDirectory))
        throw errors.physicalPathNotExists(settings.websiteDirectory);
    let server = new maishu_node_web_server_1.WebServer(settings);
    let rootDirectory = server.websiteDirectory;
    let staticFileProcessor = server.requestProcessors.staticProcessor; //.find(StaticFileProcessor);
    var javaScriptProcessor = new maishu_nws_js_1.JavaScriptProcessor();
    javaScriptProcessor.babelOptions = {
        "\\S+.js$": {
            "presets": [
                ["@babel/preset-env", {
                        "targets": { chrome: 58 }
                    }],
            ],
            plugins: [
                ["@babel/plugin-transform-modules-amd", { noInterop: true }]
            ]
        },
        "\\S+.ts$": {
            "presets": [
                ["@babel/preset-env", {
                        "targets": { chrome: 58 }
                    }]
            ],
            plugins: [
                ["@babel/plugin-proposal-decorators", { legacy: true }],
                ["@babel/plugin-transform-typescript", { isTSX: false }],
                ["@babel/plugin-transform-react-jsx", { "pragma": "React.createElement", "pragmaFrag": "React.Fragment" }],
                ["@babel/plugin-transform-modules-amd", { noInterop: true }],
            ]
        },
        "\\S+.tsx$": {
            "presets": [
                ["@babel/preset-env", {
                        "targets": { chrome: 58 }
                    }]
            ],
            plugins: [
                ["@babel/plugin-proposal-decorators", { legacy: true }],
                ["@babel/plugin-transform-typescript", { isTSX: true }],
                ["@babel/plugin-transform-react-jsx", { "pragma": "React.createElement", "pragmaFrag": "React.Fragment" }],
                ["@babel/plugin-transform-modules-amd", { noInterop: true }],
            ]
        },
    };
    server.requestProcessors.fileProcessor.processors[".js"] = javaScriptProcessor; //.add(javaScriptProcessor);
    server.requestProcessors.fileProcessor.processors[".jsx"] = javaScriptProcessor;
    server.requestProcessors.fileProcessor.processors[".ts"] = javaScriptProcessor;
    server.requestProcessors.fileProcessor.processors[".tsx"] = javaScriptProcessor;
    var json5Processor = new json5_processor_1.Json5Processor();
    server.requestProcessors.fileProcessor.processors[".json"] = json5Processor;
    server.requestProcessors.fileProcessor.processors[".json5"] = json5Processor;
    // server.requestProcessors.add(json5Processor);
    var lessProcessor = new less_processor_1.LessProcessor();
    server.requestProcessors.fileProcessor.processors[".less"] = lessProcessor;
    server.requestProcessors.fileProcessor.processors[".css"] = lessProcessor;
    server.requestProcessors.fileProcessor.processors[".scss"] = lessProcessor;
    // server.requestProcessors.add(lessProcessor);
    if (settings.headers) {
        var headersProcessor = server.requestProcessors.headersProcessor; //.find(HeadersProcessor);
        console.assert(headersProcessor != null, "Can not find headers processor.");
        headersProcessor.headers = headersProcessor.headers || {};
        for (let key in settings.headers) {
            headersProcessor.headers[key] = settings.headers[key];
        }
        let outputError = server.outputError;
        server.outputError = function (err, res) {
            for (let key in settings.headers) {
                res.setHeader(key, settings.headers[key]);
            }
            outputError.apply(this, [err, res]);
        };
    }
    if (settings.virtualPaths) {
        for (let virtualPath in settings.virtualPaths) {
            let physicalPath = settings.virtualPaths[virtualPath];
            if (virtualPath[0] != "/")
                virtualPath = "/" + virtualPath;
            server.websiteDirectory.setPath(virtualPath, physicalPath);
        }
    }
    if (settings.proxy) {
        let proxyProcessor = server.requestProcessors.proxyProcessor; //.find(ProxyProcessor);
        proxyProcessor.proxyTargets = settings.proxy;
    }
    let mvcProcessor = new maishu_nws_mvc_1.MVCRequestProcessor();
    server.requestProcessors.fileProcessor.processors[""] = mvcProcessor;
    // mvcProcessor.priority = processorPriorities.ProxyRequestProcessor + 10;
    // server.requestProcessors.add(mvcProcessor);
    settings.controllerDirectory = settings.controllerDirectory || "controllers";
    mvcProcessor.controllerDirectories = [settings.controllerDirectory];
    mvcProcessor.contextData = settings.contextData || settings.serverContextData;
    load_plugins_1.loadPlugins(rootDirectory, logger, server);
    if (settings.processors != null) {
        let requestProcessors = server.requestProcessors.items;
        for (let i = 0; i < requestProcessors.length; i++) {
            let requestProcessor = requestProcessors[i];
            let name = requestProcessor.constructor.name;
            let shortName = null;
            if (name.endsWith("Processor")) {
                shortName = name.substr(0, name.length - "Processor".length);
            }
            let processorProperties = null; //= settings.processors[name];
            if (shortName != null) {
                processorProperties = settings.processors[shortName];
            }
            if (processorProperties == null) {
                processorProperties = settings.processors[name];
            }
            processorProperties = processorProperties || {};
            for (let prop in processorProperties) {
                if (requestProcessor[prop]) {
                    requestProcessor[prop] = processorProperties[prop];
                }
            }
        }
    }
    if (mode == "mvc") {
        let staticDir = rootDirectory.findDirectory("public") || rootDirectory.findDirectory("static");
        let nodeModulesDir = rootDirectory.findDirectory("node_modules");
        if (staticDir != null && nodeModulesDir != null) {
            staticDir.setPath("node_modules", nodeModulesDir.physicalPath);
        }
        staticFileProcessor.staticPath = rootDirectory.findDirectory("public") != null ? "public" : "static";
        javaScriptProcessor.directoryPath = staticFileProcessor.staticPath === null ? undefined : staticFileProcessor.staticPath;
        lessProcessor.directoryPath = staticFileProcessor.staticPath;
    }
    return server;
}
exports.startServer = startServer;
